# -*- coding: utf-8 -*-

import logging

import click
from click.testing import CliRunner

import click_logging

import pytest


test_logger = logging.getLogger(__name__)
click_logging.basic_config(test_logger)
test_logger.level = logging.INFO


@pytest.fixture
def runner():
    return CliRunner()


def test_basic(runner):
    @click.command()
    def cli():
        test_logger.info('hey')
        test_logger.error('damn')

    result = runner.invoke(cli, catch_exceptions=False)
    assert not result.exception
    assert result.output == 'hey\nerror: damn\n'


def test_multilines(runner):
    @click.command()
    def cli():
        test_logger.warning("""
            Lorem ipsum dolor sit amet,
            consectetur adipiscing elit,
            sed do eiusmod tempor incididunt""")

    result = runner.invoke(cli, catch_exceptions=False)
    assert not result.exception
    assert result.output == (
        'warning: \n'
        'warning:             Lorem ipsum dolor sit amet,\n'
        'warning:             consectetur adipiscing elit,\n'
        'warning:             sed do eiusmod tempor incididunt\n')


def test_unicode(runner):
    @click.command()
    def cli():
        test_logger.error(u"""
            ❤️ 💔 💌 💕 💞 💓 💗 💖 💘
            💝 💟 💜 💛 💚 💙""")

    result = runner.invoke(cli, catch_exceptions=False)
    assert not result.exception
    assert result.output == (
        'error: \n'
        u'error:             ❤️ 💔 💌 💕 💞 💓 💗 💖 💘\n'
        u'error:             💝 💟 💜 💛 💚 💙\n')


def test_weird_types_log(runner):
    @click.command()
    def cli():
        test_logger.error(42)
        test_logger.error('42')
        test_logger.error(b'42')
        test_logger.error(u'42')

    result = runner.invoke(cli, catch_exceptions=False)
    assert not result.exception
    assert set(result.output.splitlines()) <= set(('error: 42', 'error: b\'42\''))


def test_early_logging(runner):
    i = None

    def callback(context, param, value):
        test_logger.debug('catch me {}!'.format(i))

    @click.command()
    @click_logging.simple_verbosity_option(test_logger)
    @click.option('--config', is_eager=True, default=None, expose_value=False,
                  callback=callback)
    def cli():
        test_logger.debug('hello')

    for i in range(2):
        result = runner.invoke(cli, ['-v', 'debug'], catch_exceptions=False)
        assert 'debug: hello' in result.output
        assert 'debug: catch me {}!'.format(i) in result.output


def test_logging_args(runner):
    @click.command()
    @click_logging.simple_verbosity_option(test_logger)
    def cli():
        test_logger.debug('hello %s', 'world')

    result = runner.invoke(cli, ['-v', 'debug'])
    assert 'debug: hello world' in result.output



#!/usr/bin/env python3

import re
import os
import sys

from setuptools import setup, find_packages
try:
    from semantic_release import setup_hook

    setup_hook(sys.argv)
except ImportError:
    pass

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    readme = f.read()

with open(os.path.join(here, 'CHANGELOG.md'), encoding='utf-8') as f:
    history = f.read()

dependency_links = []

project_dir = os.path.dirname(os.path.realpath(__file__))

requirements = os.path.join(project_dir, 'requirements.txt')
requirements_dev = os.path.join(project_dir, 'requirements-dev.txt')
requirements_ci = os.path.join(project_dir, 'requirements-ci.txt')

with open(requirements, encoding='utf-8') as f:
    install_requires = list(map(str.strip, f.read().splitlines()))[1:]

with open(requirements_dev, encoding='utf-8') as f:
    dev_require = list(filter(lambda x: '://' not in x, map(str.strip, f.read().splitlines())))[:-1]

with open(requirements_ci, encoding='utf-8') as f:
    ci_require = list(filter(lambda x: '://' not in x, map(str.strip, f.read().splitlines())))[:-1]

dependency_link_pattern = re.compile(r"(\S+:\/\/\S+)#egg=(\S+)")

dependency_links = []
install_requires_fixed = []
for req in install_requires:
    match = dependency_link_pattern.match(req)
    if match:
        install_requires_fixed.append(match.group(2))
        dependency_links.append(match.group(1))
    else:
        install_requires_fixed.append(req)
install_requires = install_requires_fixed

package_data = []

with open('click_logging/__init__.py', 'r') as f:
    version = re.search(r'^__version__\st*=\s*[\'"]([^\'"]*)[\'"]$', f.read(), re.MULTILINE).group(1)

setup(
    name='click-logging',
    version=version,
    description='Logging integration for Click',
    author='Rémi Alvergnat',
    author_email='toilal.dev@gmail.com',
    url='https://github.com/Toilal/click-logging',
    license='MIT',
    long_description=readme + '\n' * 2 + history,
    long_description_content_type='text/markdown',
    classifiers=[
        # See: https://pypi.python.org/pypi?:action=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        # List of python versions and their support status:
        # https://en.wikipedia.org/wiki/CPython#Version_history
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: System :: Logging',
    ],
    packages=find_packages(),
    dependency_links=dependency_links,
    include_package_data=True,
    zip_safe=True,
    install_requires=install_requires,
    extras_require={
        'dev': dev_require,
        'ci': ci_require
    }
)



# -*- coding: utf-8 -*-

import logging
import sys

import click

_ctx = click.get_current_context

LOGGER_KEY = __name__ + '.logger'
DEFAULT_LEVEL = logging.INFO

PY2 = sys.version_info[0] == 2

if PY2:
    text_type = unicode  # noqa
else:
    text_type = str


def _meta():
    return _ctx().meta.setdefault(LOGGER_KEY, {})


class ColorFormatter(logging.Formatter):
    def __init__(self, style_kwargs):
        self.style_kwargs = style_kwargs

    def format(self, record):
        if not record.exc_info:
            level = record.levelname.lower()
            msg = record.getMessage()
            if self.style_kwargs.get(level):
                prefix = click.style('{}: '.format(level),
                                     **self.style_kwargs[level])
                msg = '\n'.join(prefix + x for x in msg.splitlines())
            return msg
        return logging.Formatter.format(self, record)


class ClickHandler(logging.Handler):
    def __init__(self, echo_kwargs):
        super().__init__()
        self.echo_kwargs = echo_kwargs

    def emit(self, record):
        try:
            msg = self.format(record)
            level = record.levelname.lower()
            if self.echo_kwargs.get(level):
                click.echo(msg, **self.echo_kwargs[level])
            else:
                click.echo(msg)
        except Exception:
            self.handleError(record)


def _normalize_logger(logger):
    if not isinstance(logger, logging.Logger):
        logger = logging.getLogger(logger)
    return logger


def _normalize_style_kwargs(styles):
    normalized_styles = {
        'error': dict(fg='red'),
        'exception': dict(fg='red'),
        'critical': dict(fg='red'),
        'debug': dict(fg='blue'),
        'warning': dict(fg='yellow')
    }
    if styles:
        normalized_styles.update(styles)
    return normalized_styles


def _normalize_echo_kwargs(echo_kwargs):
    normamized_echo_kwargs = dict()
    if echo_kwargs:
        normamized_echo_kwargs.update(echo_kwargs)
    return normamized_echo_kwargs


def basic_config(logger=None, style_kwargs=None, echo_kwargs=None):
    '''Set up the default handler (:py:class:`ClickHandler`) and formatter
    (:py:class:`ColorFormatter`) on the given logger.'''
    logger = _normalize_logger(logger)
    style_kwargs = _normalize_style_kwargs(style_kwargs)
    echo_kwargs = _normalize_echo_kwargs(echo_kwargs)

    handler = ClickHandler(echo_kwargs)
    handler.formatter = ColorFormatter(style_kwargs)
    logger.handlers = [handler]
    logger.propagate = False

    return logger
